#pragma once

#ifndef FOOBAR_HPP
#define FOOBAR_HPP

#include "foo.hpp"
#include "bar.hpp"

void start() {
    Abc abc;
    Def def;
}

#endif // FOOBAR_HPP

#ifdef FOOBAR_MAIN

// ~> stitch <~

#endif /* FOOBAR_MAIN */
